@@ -18,5 +18,10 @@
 #  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 #  SOFTWARE.
 
-ARG_NAMING_PREFIXES = ["—", "--"]
+LONG_ARG_KEY_PREFIXES = ["—", "--"]
+ABBREVIATED_ARG_KEY_PREFIXES = ['-']
+ARG_NAMING_PREFIXES = LONG_ARG_KEY_PREFIXES + ABBREVIATED_ARG_KEY_PREFIXES
+
 ARG_VALUE_SEPARATOR_CHAR = "="
+
+QUOTE_CHARS = ['"', '\'']
