@@ -20,16 +20,18 @@
 
 import functools
 import logging
+from typing import List
 
-from telegram import ParseMode, Update
+from telegram import Update
 from telegram.ext import CallbackContext
 
 from telegram_click import CommandTarget
 from telegram_click.argument import Argument
+from telegram_click.error_handler import ErrorHandler, DEFAULT_ERROR_HANDLER
 from telegram_click.help import generate_help_message
 from telegram_click.parser import parse_telegram_command, split_command_from_args, split_command_from_target
 from telegram_click.permission.base import Permission
-from telegram_click.util import send_message, find_first, find_duplicates
+from telegram_click.util import find_first, find_duplicates
 
 LOGGER = logging.getLogger(__name__)
 
@@ -51,18 +53,17 @@ def _check_permissions(update: Update, context: CallbackContext,
 
 def _create_callback_wrapper(func: callable, help_message: str,
                              arguments: [Argument],
-                             permissions: Permission, permission_denied_message: str or None,
+                             permissions: Permission,
                              command_target: bytes,
-                             print_error: bool) -> callable:
+                             error_handlers: List[ErrorHandler]) -> callable:
     """
     Creates the wrapper function for the callback function
     :param func: the function to wrap
     :param help_message: command help message
     :param arguments: command arguments
     :param permissions: command permissions
-    :param permission_denied_message: permission denied message
     :param command_target: command target
-    :param print_error: True prints the exception stacktrace, False a general error message
+    :param error_handlers: list of error handlers
     :return: wrapper function
     """
     if not callable(func):
@@ -81,16 +82,16 @@ def _create_callback_wrapper(func: callable, help_message: str,
 
         try:
             if not _check_permissions(update, context, permissions):
+                # permission denied
                 LOGGER.debug("Permission denied in chat {} for user {} for message: {}".format(
                     chat_id,
                     update.effective_message.from_user.id,
                     message))
 
-                # send 'permission denied' message, if configured
-                if permission_denied_message is not None:
-                    send_message(bot, chat_id=chat_id, message=permission_denied_message,
-                                 parse_mode=ParseMode.MARKDOWN,
-                                 reply_to=message.message_id)
+                for handler in error_handlers:
+                    if handler.on_permission_error(context, update, permissions):
+                        break
+
                 # don't process command
                 return
 
@@ -112,16 +113,13 @@ def _create_callback_wrapper(func: callable, help_message: str,
                 # parse command and arguments
                 cmd, parsed_args = parse_telegram_command(bot.username, message.text, arguments)
             except ValueError as ex:
-                # handle exceptions that occur during permission and argument parsing
+                # error during argument parsing
                 logging.exception("Error parsing command arguments")
 
-                denied_text = "\n".join([
-                    ":exclamation: `{}`".format(str(ex)),
-                    "",
-                    help_message
-                ])
-                send_message(bot, chat_id=chat_id, message=denied_text, parse_mode=ParseMode.MARKDOWN,
-                             reply_to=message.message_id)
+                for handler in error_handlers:
+                    if handler.on_validation_error(context, update, ex, help_message):
+                        break
+
                 return
 
             # convert argument names to python param naming convention (snake-case)
@@ -129,23 +127,16 @@ def _create_callback_wrapper(func: callable, help_message: str,
             # execute wrapped function
             return func(*args, **{**kw_function_args, **kwargs})
         except Exception as ex:
-            # execute wrapped function
+            # error while executing wrapped function
             logging.exception("Error in callback")
-
-            import traceback
-            exception_text = "\n".join(list(map(lambda x: "{}:{}\n\t{}".format(x.filename, x.lineno, x.line),
-                                                traceback.extract_tb(ex.__traceback__))))
-            if print_error:
-                denied_text = ":boom: `{}`".format(exception_text)
-            else:
-                denied_text = ":boom: There was an error executing your command :worried:"
-            send_message(bot, chat_id=chat_id, message=denied_text, parse_mode=ParseMode.MARKDOWN,
-                         reply_to=message.message_id)
+            for handler in error_handlers:
+                if handler.on_execution_error(context, update, ex):
+                    break
 
     return wrapper
 
 
-def check_command_name_clashes(names: [str]):
+def check_command_name_clashes(names: List[str]):
     """
     Checks if a command name has been used multiple times and raises an exception if so
     :param names: command names added in this decorator call
@@ -163,7 +154,7 @@ def check_command_name_clashes(names: [str]):
         raise ValueError("Command names must be unique! Clashing names: {}".format(clashing))
 
 
-def check_argument_name_clashes(arguments: []):
+def check_argument_name_clashes(arguments: List[Argument]):
     """
     Checks if an argument name of a command has been used multiple times and raises an exception if so
     :param arguments: arguments of a command to check
@@ -174,23 +165,34 @@ def check_argument_name_clashes(arguments: []):
         raise ValueError("Argument names must be unique per command! Clashing arguments: {}".format(clashing))
 
 
+def check_optional_argument_after_other(command_name: str, arguments: List[Argument]):
+    """
+    Checks the order of arguments to make sure no required argument is defined after an optional one
+    :param command_name: command name the arguments belong to
+    :param arguments: arguments to check
+    """
+    optional_detected = False
+    for arg in arguments:
+        if not arg.optional and optional_detected:
+            raise AssertionError(
+                "Required argument after optional argument in command /{}: {}".format(command_name, arg.name))
+        if arg.optional:
+            optional_detected = True
+
+
 def command(name: str or [str], description: str = None,
             arguments: [Argument] = None,
             permissions: Permission = None,
-            permission_denied_message: str = None,
             command_target: bytes = CommandTarget.UNSPECIFIED | CommandTarget.SELF,
-            print_error: bool = False):
+            error_handler: ErrorHandler = None):
     """
     Decorator to turn a command handler function into a full fledged, shell like command
     :param name: Name of the command
     :param description: a short description of the command
     :param arguments: list of command argument description objects
     :param permissions: required permissions to run this command
-    :param permission_denied_message: text so send when a user is denied permission.
-                                      Set this to None to send no message at all.
     :param command_target: command targets to accept
-    :param print_error: True sends the exception message to the chat of origin,
-                        False sends a general error message
+    :param error_handler: a customized error handler
     """
     from telegram_click import COMMAND_LIST
 
@@ -200,6 +202,7 @@ def command(name: str or [str], description: str = None,
 
     check_command_name_clashes(name)
     check_argument_name_clashes(arguments)
+    check_optional_argument_after_other(name, arguments)
 
     help_message = generate_help_message(name, description, arguments)
 
@@ -213,14 +216,19 @@ def command(name: str or [str], description: str = None,
         }
     )
 
+    error_handlers = [DEFAULT_ERROR_HANDLER]
+    if error_handler is not None:
+        error_handlers.insert(0, error_handler)
+
     def callback_decorator(func: callable):
         """
         Callback decorator function
         :param func: the function to wrap
         :return: wrapper function
         """
-        return _create_callback_wrapper(func, help_message, arguments, permissions, permission_denied_message,
-                                        command_target, print_error)
+        return _create_callback_wrapper(
+            func, help_message, arguments, permissions,
+            command_target, error_handlers)
 
     return callback_decorator
 
